@@ -6,11 +6,12 @@
 #pragma once
 
 #include <gtk/gtk.h>
+#include "monitor/monitor.h"
 #include "layersurface.h"
 
 #define PHOSH_TYPE_LOCKSHIELD (phosh_lockshield_get_type())
 
 G_DECLARE_FINAL_TYPE (PhoshLockshield, phosh_lockshield, PHOSH, LOCKSHIELD, PhoshLayerSurface)
 
-GtkWidget * phosh_lockshield_new (gpointer layer_shell,
-                                  gpointer wl_output);
+GtkWidget *phosh_lockshield_new (struct zwlr_layer_shell_v1 *layer_shell,
+                                 PhoshMonitor               *monitor);
