@@ -1207,20 +1207,20 @@ take_screenshot (GSimpleAction *action, GVariant *param, gpointer data)
 static void
 add_keybindings (PhoshScreenshotManager *self)
 {
-  g_auto (GStrv) bindings = NULL;
+  g_autoptr (GStrvBuilder) builder = g_strv_builder_new ();
   g_autoptr (GArray) actions = g_array_new (FALSE, TRUE, sizeof (GActionEntry));
 
-  bindings = g_settings_get_strv (self->settings, KEYBINDING_KEY_SCREENSHOT);
-  for (int i = 0; i < g_strv_length (bindings); i++) {
-    GActionEntry entry = { .name = bindings[i], .activate = take_screenshot };
-    g_array_append_val (actions, entry);
-  }
+  PHOSH_UTIL_BUILD_KEYBINDING (actions,
+                               builder,
+                               self->settings,
+                               KEYBINDING_KEY_SCREENSHOT,
+                               take_screenshot);
 
   phosh_shell_add_global_keyboard_action_entries (phosh_shell_get_default (),
                                                   (GActionEntry *)actions->data,
                                                   actions->len,
                                                   self);
-  self->action_names = g_steal_pointer (&bindings);
+  self->action_names = g_strv_builder_end (builder);
 }
 
 
